/**
 * 
 */
package org.processmining.models.performancemeasurement.dataelements;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.models.fuzzyperformancediagram.FPD;
import org.processmining.models.fuzzyperformancediagram.FPDNode;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version May 6, 2009
 */
public class TwoFPDNodesPerformanceData {
	/**
	 * performance values of two nodes are stored as follows: - 1st FPDNode :
	 * source FPDNode - 2nd FPDNode : target FPDNode - List of Long : list of
	 * elapsed time between source and and target FPDNode.
	 */
	private Map<FPDNode, Map<FPDNode, List<Long>>> performances;

	/**
	 * frequency of cases where a node occurs after another, as well as the
	 * number of cases which fits the model.
	 */
	private Map<FPDNode, Map<FPDNode, Pair<Integer, Integer>>> pairFrequencyAndFittingCases;

	/**
	 * Constructor
	 */
	public TwoFPDNodesPerformanceData(FPD fpd) {
		// initiate twoNodesPerformance
		performances = new HashMap<FPDNode, Map<FPDNode, List<Long>>>();
		pairFrequencyAndFittingCases = new HashMap<FPDNode, Map<FPDNode, Pair<Integer, Integer>>>();
		for (FPDNode node : fpd.getNodes()) {
			Map<FPDNode, List<Long>> mapList = new HashMap<FPDNode, List<Long>>();
			performances.put(node, mapList);

			Map<FPDNode, Pair<Integer, Integer>> mapInteger = new HashMap<FPDNode, Pair<Integer, Integer>>();
			pairFrequencyAndFittingCases.put(node, mapInteger);
		}
	}

	/**
	 * @return the performances
	 */
	public Map<FPDNode, Map<FPDNode, List<Long>>> getPerformances() {
		return performances;
	}

	/**
	 * @param performances
	 *            the performances to set
	 */
	public void setPerformances(Map<FPDNode, Map<FPDNode, List<Long>>> performances) {
		this.performances = performances;
	}

	/**
	 * @return the pairFrequencyAndFittingCases
	 */
	public Map<FPDNode, Map<FPDNode, Pair<Integer, Integer>>> getPairFrequencyAndFittingCases() {
		return pairFrequencyAndFittingCases;
	}

	/**
	 * @param pairFrequencyAndFittingCases
	 *            the pairFrequencyAndFittingCases to set
	 */
	public void setPairFrequencyAndFittingCases(
			Map<FPDNode, Map<FPDNode, Pair<Integer, Integer>>> pairFrequencyAndFittingCases) {
		this.pairFrequencyAndFittingCases = pairFrequencyAndFittingCases;
	}

	/**
	 * Add time distance between sourceNode to targetNode in a case
	 * 
	 * @param sourceNode
	 * @param targetNode
	 * @param performance
	 */
	public void addPerformanceValue(FPDNode sourceNode, FPDNode targetNode, Long performance) {
		Map<FPDNode, List<Long>> map = performances.get(sourceNode);
		if (map.containsKey(targetNode)) {
			// add to existing list
			map.get(targetNode).add(performance);
		} else {
			// create new one
			List<Long> list = new LinkedList<Long>();
			list.add(performance);
			map.put(targetNode, list);
		}
	}

	/**
	 * Update frequency
	 * 
	 * @param sourceNode
	 * @param targetNode
	 * @param increaseFreq
	 * @param increaseFitCase
	 */
	public void updateFrequencyAndFittingCase(FPDNode sourceNode, FPDNode targetNode, boolean increaseFitCase) {
		Map<FPDNode, Pair<Integer, Integer>> mapOfPair = pairFrequencyAndFittingCases.get(sourceNode);
		if (mapOfPair.containsKey(targetNode)) {
			// only have to increase current calculation
			Pair<Integer, Integer> oldPair = mapOfPair.get(targetNode);
			Pair<Integer, Integer> newPair = new Pair<Integer, Integer>(oldPair.getFirst() + 1,
					increaseFitCase ? oldPair.getSecond() + 1 : oldPair.getSecond());
			mapOfPair.put(targetNode, newPair);
		} else {
			// create new one
			Pair<Integer, Integer> newPair = new Pair<Integer, Integer>(1, increaseFitCase ? 1 : 0);
			mapOfPair.put(targetNode, newPair);
		}
	}

	/**
	 * return list of elapsed time between two node execution
	 * 
	 * @param sourceNode
	 * @param targetNode
	 * @return
	 */
	public List<Long> getTwoNodesPerformance(FPDNode sourceNode, FPDNode targetNode) {
		if (performances.containsKey(sourceNode)) {
			Map<FPDNode, List<Long>> temp = performances.get(sourceNode);
			if (temp.containsKey(targetNode)) {
				return temp.get(targetNode);
			}
		}
		return null;
	}

	/**
	 * return pair of integer, each has a value: - equal to the number of cases
	 * which has both sourceNode and targetNode, and - equal to number of cases
	 * which has both sourceNode and targetNode, and also fits a flow of an FPD
	 * 
	 * @param sourceNode
	 * @param targetNode
	 * @return
	 */
	public Pair<Integer, Integer> getPairFrequencyAndFittingCases(FPDNode sourceNode, FPDNode targetNode) {
		if (pairFrequencyAndFittingCases.containsKey(sourceNode)) {
			Map<FPDNode, Pair<Integer, Integer>> temp = pairFrequencyAndFittingCases.get(sourceNode);
			if (temp.containsKey(targetNode)) {
				return temp.get(targetNode);
			}
		}
		return null;
	}
}
