/**
 * 
 */
package org.processmining.models.fuzzyperformancediagram;

import java.awt.Color;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.AttributeMap.ArrowType;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphEdge;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 19, 2009
 */
public abstract class FPDEdge<S extends FPDNode, T extends FPDNode> extends AbstractDirectedGraphEdge<S, T> {
	// actual information 
	private int frequency = 0;
	private double avgMovingTime = 0.00;
	private int violatingFrequency = 0;

	// performance boundary 
	private double movingTimeBoundaryLow = 0;
	private double movingTimeBoundaryHigh = 0;
	private int divider = 1;

	/**
	 * GUI related variables
	 */
	public static final int MAXWIDTH = 5;

	// obtained after all traces are replayed
	private int maxEdgeFreq = 0;

	/**
	 * Default constructor
	 * 
	 * @param source
	 * @param target
	 */
	public FPDEdge(S source, T target) {
		super(source, target);
		getAttributeMap().put(AttributeMap.SHOWLABEL, false);
		getAttributeMap().put(AttributeMap.EDGEEND, ArrowType.ARROWTYPE_TECHNICAL);
	}

	/**
	 * get frequency of an edge
	 * 
	 * @return
	 */
	public int getFrequency() {
		return frequency;
	}

	/**
	 * set frequency value of an edge
	 * 
	 * @param frequency
	 */
	public void setFrequency(int frequency) {
		this.frequency = frequency;
	}

	/**
	 * increment frequency value of an edge
	 */
	public void incrementFrequency() {
		this.frequency++;
	}

	/**
	 * decrement frequency value of an edge
	 */
	public void decrementFrequency() {
		this.frequency--;
	}

	/**
	 * get the number of violating flow frequency
	 * 
	 * @return
	 */
	public int getViolatingFrequency() {
		return violatingFrequency;
	}

	/**
	 * set the number of violating flow frequency
	 * 
	 * @param violatingFrequency
	 */
	public void setViolatingFrequency(int violatingFrequency) {
		this.violatingFrequency = violatingFrequency;
	}

	/**
	 * increment the number of violating flow frequency by 1
	 */
	public void incrementViolatingFrequency() {
		this.violatingFrequency++;
	}

	/**
	 * decrement the number of violating flow frequency by 1
	 */
	public void decrementViolatingFrequency() {
		this.violatingFrequency--;
	}

	/**
	 * return average moving time for this edge
	 * 
	 * @return
	 */
	public double getAvgMovingTime() {
		return avgMovingTime;
	}

	/**
	 * set the value of average moving time for this edge
	 * 
	 * @param avgMovingTime
	 */
	public void setAvgMovingTime(double avgMovingTime) {
		this.avgMovingTime = avgMovingTime;
	}

	/**
	 * @return the movingTimeBoundaryLow
	 */
	public double getMovingTimeBoundaryLow() {
		return movingTimeBoundaryLow;
	}

	/**
	 * @param movingTimeBoundaryLow
	 *            the movingTimeBoundaryLow to set
	 */
	public void setMovingTimeBoundaryLow(double movingTimeBoundaryLow) {
		this.movingTimeBoundaryLow = movingTimeBoundaryLow;
	}

	/**
	 * @return the movingTimeBoundaryHigh
	 */
	public double getMovingTimeBoundaryHigh() {
		return movingTimeBoundaryHigh;
	}

	/**
	 * @param movingTimeBoundaryHigh
	 *            the movingTimeBoundaryHigh to set
	 */
	public void setMovingTimeBoundaryHigh(double movingTimeBoundaryHigh) {
		this.movingTimeBoundaryHigh = movingTimeBoundaryHigh;
	}

	/**
	 * return string of information about this edge
	 * 
	 * @return
	 */
	public String getInfo() {
		StringBuffer result = new StringBuffer();
		result.append("EDGE INFO: \nEdge label : ");
		result.append(getLabel());
		result.append("\n");
		result.append("Frequency: ");
		result.append(getFrequency());
		result.append("\n");
		result.append("Violating Frequency: ");
		result.append(getViolatingFrequency());
		result.append("\n");
		result.append("Average moving time: ");
		result.append(getAvgMovingTime());
		result.append("\n");

		// performance boundary 
		result.append("Moving time boundary low: ");
		result.append(movingTimeBoundaryLow);
		result.append("\n");
		result.append("Moving time boundary high: ");
		result.append(movingTimeBoundaryHigh);
		result.append("\n");
		return result.toString();
	}

	/**
	 * update GUI appearance of this edge
	 */
	public void updateEdgeInterface() {
		if (getFrequency() > 0) {
			getAttributeMap().put(AttributeMap.LINEWIDTH, new Float(1 + Math.log(Math.E) * Math.log(getFrequency())));
		} else {
			getAttributeMap().put(AttributeMap.LINEWIDTH, new Float(0));
		}
		getAttributeMap().put(AttributeMap.EDGECOLOR,
				getPerformanceColor(avgMovingTime, movingTimeBoundaryLow, movingTimeBoundaryHigh));
		getAttributeMap().put(
				AttributeMap.LABEL,
				"[Avg move time:" + String.valueOf(getAvgMovingTime() / divider) + "][Freq:"
						+ String.valueOf(getFrequency()) + "]");
	}

	/**
	 * method to determine performance color based on lower and high boundary
	 * 
	 * @param time
	 * @param timeLow
	 * @param timeHigh
	 * @return
	 */
	private Color getPerformanceColor(double time, double timeLow, double timeHigh) {
		if (time <= timeLow) {
			return Color.GREEN;
		} else if (time <= timeHigh) {
			return Color.YELLOW;
		} else {
			return Color.RED;
		}
	}

	/**
	 * @return the maxEdgeFreq
	 */
	public int getMaxEdgeFreq() {
		return maxEdgeFreq;
	}

	/**
	 * @param maxEdgeFreq
	 *            the maxEdgeFreq to set
	 */
	public void setMaxEdgeFreq(int maxEdgeFreq) {
		this.maxEdgeFreq = maxEdgeFreq;
	}

	/**
	 * @return the divider
	 */
	public int getDivider() {
		return divider;
	}

	/**
	 * @param divider
	 *            the divider to set
	 */
	public void setDivider(int divider) {
		this.divider = divider;
	}

}
