/**
 * 
 */
package org.processmining.models.fuzzyperformancediagram;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.swing.JLabel;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.connections.GraphLayoutConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.graphbased.directed.utils.GraphIterator;
import org.processmining.models.graphbased.directed.utils.GraphIterator.EdgeAcceptor;
import org.processmining.models.graphbased.directed.utils.GraphIterator.NodeAcceptor;
import org.processmining.models.shapes.Decorated;
import org.processmining.models.shapes.FPDNodeShape;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 19, 2009
 */
public abstract class FPDNode extends AbstractDirectedGraphNode implements Decorated {
	/**
	 * DATA VARIABLES
	 */
	// linkage to graph
	private final AbstractDirectedGraph<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>> graph;

	// event classes which are mapped to this node
	// must be unique
	private List<XEventClass> eventClasses = new LinkedList<XEventClass>();
	private List<Integer> freqEventClasses = new LinkedList<Integer>(); // this determines frequency number indicator

	// factual information
	private int ANDJOINrf = 0;
	private int ORJOINrf = 0;
	private int XORJOINrf = 0;
	private int ANDSPLITrf = 0;
	private int ORSPLITrf = 0;
	private int XORSPLITrf = 0;

	private double avgThroughput = -1;
	private double waitingTime = -1;
	private double synchronizationTime = -1;

	// performance boundary 
	private double throughputTimeLow = 0;
	private double throughputTimeHigh = 0;
	private double waitingTimeLow = 0;
	private double waitingTimeHigh = 0;
	private double synchronizationTimeLow = 0;
	private double synchronizationTimeHigh = 0;

	private int freqCaseHasThisNode = 0; // this determines the filling of the node

	// initialization/termination freq
	private int initializationFreq = 0;
	private int terminationFreq = 0;
	private int activationFreq = 0;

	// related to resource
	private int numberOfPerformer = 0;

	// obtained after all traces are replayed
	private int numberOfCases = 0;
	private int maxFreqEventClass = 0;

	// utility
	private final NumberFormat nf;
	private int divider = 1;

	/**
	 * GUI RELATED VARIABLES
	 */
	public static final int PADDINGFROMBOXTOTEXT = 10;
	public static final int TEXTHEIGHT = 6;
	public static final int BRANCHINGBOXWIDTH = 50;
	public static final int STANDARDWIDTH = 300;
	public static final int STANDARDHEIGHT = 100;
	public static final int MINHEIGHT = 60;
	public static final int MAXLABELCHAR = 21;

	/**
	 * Default constructor
	 * 
	 * @param graph
	 * @param label
	 */
	public FPDNode(AbstractDirectedGraph<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>> graph, String label) {
		super();
		this.graph = graph;

		// init utility
		nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);

		getAttributeMap().put(AttributeMap.SHAPE, new FPDNodeShape(this, PADDINGFROMBOXTOTEXT, BRANCHINGBOXWIDTH));
		getAttributeMap().put(AttributeMap.RESIZABLE, false);
		getAttributeMap().put(AttributeMap.FILLCOLOR, new Color(200, 200, 200));
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(STANDARDWIDTH, MINHEIGHT)); // default value
		getAttributeMap().put(AttributeMap.LABEL, label);
		getAttributeMap().put(AttributeMap.SHOWLABEL, false);
		getAttributeMap().put(AttributeMap.EDGESTARTFILLED, true);
		getAttributeMap().put(AttributeMap.EDGEENDFILLED, true);
	}

	/**
	 * get frequency of ANDJOIN
	 * 
	 * @return
	 */
	public int getANDJOINrf() {
		return ANDJOINrf;
	}

	/**
	 * set frequency of ANDJOIN
	 * 
	 * @param nrf
	 */
	public void setANDJOINrf(int nrf) {
		ANDJOINrf = nrf;
	}

	/**
	 * increment frequency of ANDJOIN by 1
	 */
	public void incrementANDJOINrf() {
		ANDJOINrf++;
	}

	/**
	 * decrement frequency of ANDJOIN by 1
	 */
	public void decrementANDJOINrf() {
		ANDJOINrf--;
	}

	/**
	 * get frequency of ORJOIN
	 * 
	 * @return
	 */
	public int getORJOINrf() {
		return ORJOINrf;
	}

	/**
	 * set frequency of ORJOIN
	 * 
	 * @param nrf
	 */
	public void setORJOINrf(int nrf) {
		ORJOINrf = nrf;
	}

	/**
	 * increment frequency of ORJOIN by 1
	 */
	public void incrementORJOINrf() {
		ORJOINrf++;
	}

	/**
	 * decrement frequency of ORJOIN by 1
	 */
	public void decrementORJOINrf() {
		ORJOINrf--;
	}

	/**
	 * get frequency of XORJOIN
	 * 
	 * @return
	 */
	public int getXORJOINrf() {
		return XORJOINrf;
	}

	/**
	 * set frequency of XORJOIN
	 * 
	 * @param nrf
	 */
	public void setXORJOINrf(int nrf) {
		XORJOINrf = nrf;
	}

	/**
	 * increment frequency of XORJOIN by 1
	 */
	public void incrementXORJOINrf() {
		XORJOINrf++;
	}

	/**
	 * decrement frequency of XORJOIN by 1
	 */
	public void decrementXORJOINrf() {
		XORJOINrf--;
	}

	/**
	 * get frequency of ANDSPLIT
	 * 
	 * @return
	 */
	public int getANDSPLITrf() {
		return ANDSPLITrf;
	}

	/**
	 * set frequency of ANDSPLIT
	 * 
	 * @param trf
	 */
	public void setANDSPLITrf(int trf) {
		ANDSPLITrf = trf;
	}

	/**
	 * increment frequency of ANDSPLIT by 1
	 */
	public void incrementANDSPLITrf() {
		ANDSPLITrf++;
	}

	/**
	 * decrement frequency of ANDSPLIT by 1
	 */
	public void decrementANDSPLITrf() {
		ANDSPLITrf--;
	}

	/**
	 * get frequency of ORSPLIT
	 * 
	 * @return
	 */
	public int getORSPLITrf() {
		return ORSPLITrf;
	}

	/**
	 * set frequency of ORSPLIT
	 * 
	 * @param trf
	 */
	public void setORSPLITrf(int trf) {
		ORSPLITrf = trf;
	}

	/**
	 * increment frequency of ORSPLIT
	 */
	public void incrementORSPLITrf() {
		ORSPLITrf++;
	}

	/**
	 * decrement frequency of ORSPLIT
	 */
	public void decrementORSPLITrf() {
		ORSPLITrf--;
	}

	/**
	 * get frequency of XORSPLIT
	 * 
	 * @return
	 */
	public int getXORSPLITrf() {
		return XORSPLITrf;
	}

	/**
	 * set frequency of XORSPLIT
	 * 
	 * @param trf
	 */
	public void setXORSPLITrf(int trf) {
		XORSPLITrf = trf;
	}

	/**
	 * increment frequency of XORSPLIT by 1
	 */
	public void incrementXORSPLITrf() {
		XORSPLITrf++;
	}

	/**
	 * decrement frequency of XORSPLIT by 1
	 */
	public void decrementXORSPLITrf() {
		XORSPLITrf--;
	}

	/**
	 * get average throughput of this node
	 * 
	 * @return
	 */
	public double getAvgThroughput() {
		return avgThroughput;
	}

	/**
	 * set average throughput of this node
	 * 
	 * @param avgThroughput
	 */
	public void setAvgThroughput(double avgThroughput) {
		this.avgThroughput = avgThroughput;
	}

	/**
	 * get average waiting time of this node
	 * 
	 * @return
	 */
	public double getWaitingTime() {
		return waitingTime;
	}

	/**
	 * set average waiting time of this node
	 * 
	 * @param waitingTime
	 */
	public void setWaitingTime(double waitingTime) {
		this.waitingTime = waitingTime;
	}

	/**
	 * get average synchronization time of this node
	 * 
	 * @return
	 */
	public double getSynchronizationTime() {
		return synchronizationTime;
	}

	/**
	 * set average synchronization time of this node
	 * 
	 * @param synchronizationTime
	 */
	public void setSynchronizationTime(double synchronizationTime) {
		this.synchronizationTime = synchronizationTime;
	}

	/**
	 * @return the eventClasses
	 */
	public List<XEventClass> getEventClasses() {
		return eventClasses;
	}

	/**
	 * @param eventClasses
	 *            the eventClasses to set
	 */
	public void setEventClasses(List<XEventClass> eventClasses) {
		this.eventClasses = eventClasses;
	}

	/**
	 * @return the freqEventClasses
	 */
	public List<Integer> getFreqEventClasses() {
		return freqEventClasses;
	}

	/**
	 * @param freqEventClasses
	 *            the freqEventClasses to set
	 */
	public void setFreqEventClasses(List<Integer> freqEventClasses) {
		this.freqEventClasses = freqEventClasses;
	}

	/**
	 * @return the freqCaseHasThisNode
	 */
	public int getFreqCaseHasThisNode() {
		return freqCaseHasThisNode;
	}

	/**
	 * @param freqCaseHasThisNode
	 *            the freqCaseHasThisNode to set
	 */
	public void setFreqCaseHasThisNode(int freqCaseHasThisNode) {
		this.freqCaseHasThisNode = freqCaseHasThisNode;
	}

	/**
	 * increment frequency of cases in which this node occurs
	 */
	public void incrementFreqCaseHasThisNode() {
		freqCaseHasThisNode++;
	}

	/**
	 * decrement frequency of cases in which this node occurs
	 */
	public void decrementFreqCaseHasThisNode() {
		freqCaseHasThisNode--;
	}

	/**
	 * @return the throughputTimeLow
	 */
	public double getThroughputTimeLow() {
		return throughputTimeLow;
	}

	/**
	 * @param throughputTimeLow
	 *            the throughputTimeLow to set
	 */
	public void setThroughputTimeLow(double throughputTimeLow) {
		this.throughputTimeLow = throughputTimeLow;
	}

	/**
	 * @return the throughputTimeHigh
	 */
	public double getThroughputTimeHigh() {
		return throughputTimeHigh;
	}

	/**
	 * @param throughputTimeHigh
	 *            the throughputTimeHigh to set
	 */
	public void setThroughputTimeHigh(double throughputTimeHigh) {
		this.throughputTimeHigh = throughputTimeHigh;
	}

	/**
	 * @return the waitingTimeLow
	 */
	public double getWaitingTimeLow() {
		return waitingTimeLow;
	}

	/**
	 * @param waitingTimeLow
	 *            the waitingTimeLow to set
	 */
	public void setWaitingTimeLow(double waitingTimeLow) {
		this.waitingTimeLow = waitingTimeLow;
	}

	/**
	 * @return the waitingTimeHigh
	 */
	public double getWaitingTimeHigh() {
		return waitingTimeHigh;
	}

	/**
	 * @param waitingTimeHigh
	 *            the waitingTimeHigh to set
	 */
	public void setWaitingTimeHigh(double waitingTimeHigh) {
		this.waitingTimeHigh = waitingTimeHigh;
	}

	/**
	 * @return the synchronizationTimeLow
	 */
	public double getSynchronizationTimeLow() {
		return synchronizationTimeLow;
	}

	/**
	 * @param synchronizationTimeLow
	 *            the synchronizationTimeLow to set
	 */
	public void setSynchronizationTimeLow(double synchronizationTimeLow) {
		this.synchronizationTimeLow = synchronizationTimeLow;
	}

	/**
	 * @return the synchronizationTimeHigh
	 */
	public double getSynchronizationTimeHigh() {
		return synchronizationTimeHigh;
	}

	/**
	 * @param synchronizationTimeHigh
	 *            the synchronizationTimeHigh to set
	 */
	public void setSynchronizationTimeHigh(double synchronizationTimeHigh) {
		this.synchronizationTimeHigh = synchronizationTimeHigh;
	}

	/**
	 * return the graph which this node is belong to
	 */
	public AbstractDirectedGraph<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>> getGraph() {
		return graph;
	}

	/**
	 * get Label of the node
	 */
	public String getLabel() {
		return getAttributeMap().get(AttributeMap.LABEL).toString();
	}

	/**
	 * set Label of the node
	 * 
	 * @param newLabel
	 */
	public void setLabel(String newLabel) {
		getAttributeMap().remove(AttributeMap.LABEL);
		getAttributeMap().put(AttributeMap.LABEL, newLabel);
	}

	/**
	 * Add a new eventClass to this node
	 * 
	 * @param newEventClass
	 * @return
	 */
	public boolean addNodeMapping(XEventClass newEventClass) {
		if (!eventClasses.contains(newEventClass)) {
			freqEventClasses.add(0);
			eventClasses.add(newEventClass);
			return true;
		}
		return false;
	}

	/**
	 * Add all eventclass ini setOfEventClass to this node
	 * 
	 * @param setOfEventClass
	 * @return
	 */
	public void addAllNodeMapping(Set<XEventClass> setOfEventClass) {
		for (XEventClass eventClass : setOfEventClass) {
			if (!eventClasses.contains(setOfEventClass)) {
				eventClasses.add(eventClass);
				freqEventClasses.add(0);
			}
		}
	}

	/**
	 * Remove an event class from this node
	 * 
	 * @param removedEventClass
	 * @return
	 */
	public void removeNodeMapping(XEventClass removedEventClass) {
		int index = eventClasses.indexOf(removedEventClass);
		eventClasses.remove(index);
		freqEventClasses.remove(index);
	}

	/**
	 * Remove all mapping of this node
	 */
	public void removeAllMapping() {
		eventClasses.clear();
		freqEventClasses.clear();
	}

	/**
	 * set frequency of event class
	 * 
	 * @param eventClass
	 * @param frequency
	 * @return
	 */
	public boolean setEventClassFrequency(XEventClass eventClass, int frequency) {
		int index = eventClasses.indexOf(eventClass);
		if (index > 0) {
			freqEventClasses.set(index, frequency);
			return true;
		} else {
			return false;
		}
	}

	/**
	 * get frequency of event class
	 * 
	 * @param eventClass
	 * @param frequency
	 * @return
	 */
	public int getEventClassFrequency(XEventClass eventClass, int frequency) {
		int index = eventClasses.indexOf(eventClass);
		if (index > 0) {
			return freqEventClasses.get(index);
		} else {
			return -1;
		}
	}

	/**
	 * get all successor for this node
	 * 
	 * @return
	 */
	public Collection<FPDNode> getVisibleSuccessors() {

		final NodeAcceptor<FPDNode> nodeAcceptor = new NodeAcceptor<FPDNode>() {
			public boolean acceptNode(FPDNode node, int depth) {
				return (depth != 0);
			}
		};

		Collection<FPDNode> nodes = GraphIterator.getDepthFirstSuccessors(this, getGraph(),
				new EdgeAcceptor<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>>() {

					public boolean acceptEdge(FPDEdge<? extends FPDNode, ? extends FPDNode> edge, int depth) {
						return !nodeAcceptor.acceptNode(edge.getSource(), depth);
					}
				}, nodeAcceptor);

		return Arrays.asList(nodes.toArray(new FPDNode[0]));
	}

	/**
	 * get all predecessor for this node
	 * 
	 * @return
	 */
	public Collection<FPDNode> getVisiblePredecessors() {
		final NodeAcceptor<FPDNode> nodeAcceptor = new NodeAcceptor<FPDNode>() {
			public boolean acceptNode(FPDNode node, int depth) {
				return (depth != 0);
			}
		};

		Collection<FPDNode> transitions = GraphIterator.getDepthFirstPredecessors(this, getGraph(),
				new EdgeAcceptor<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>>() {

					public boolean acceptEdge(FPDEdge<? extends FPDNode, ? extends FPDNode> edge, int depth) {
						return !nodeAcceptor.acceptNode(edge.getTarget(), depth);
					}
				}, nodeAcceptor);
		return Arrays.asList(transitions.toArray(new FPDNode[0]));
	}

	/**
	 * return info for this node
	 * 
	 * @return
	 */
	public String getInfo() {
		StringBuffer result = new StringBuffer();
		result.append("NODE INFO: \n");
		for (XEventClass eventClass : eventClasses) {
			result.append(eventClass.toString());
			result.append("\n");
		}

		result.append("AND Join: ");
		result.append(ANDJOINrf);
		result.append("\n");
		result.append("OR Join: ");
		result.append(ORJOINrf);
		result.append("\n");
		result.append("XOR Join: ");
		result.append(XORJOINrf);
		result.append("\n");
		result.append("AND Split: ");
		result.append(ANDSPLITrf);
		result.append("\n");
		result.append("OR Split: ");
		result.append(ORSPLITrf);
		result.append("\n");
		result.append("XOR Split: ");
		result.append(XORSPLITrf);
		result.append("\n");

		result.append("Average throughput: ");
		result.append(avgThroughput);
		result.append("\n");
		result.append("Waiting time: ");
		result.append(waitingTime);
		result.append("\n");
		result.append("Synchronization time: ");
		result.append(synchronizationTime);
		result.append("\n");

		// performance boundary 
		result.append("Throughput time high: ");
		result.append(throughputTimeHigh);
		result.append("\n");
		result.append("Throughput time low: ");
		result.append(throughputTimeLow);
		result.append("\n");
		result.append("Waiting time high: ");
		result.append(waitingTimeHigh);
		result.append("\n");
		result.append("Waiting time low: ");
		result.append(waitingTimeLow);
		result.append("\n");
		result.append("Synchronization time high: ");
		result.append(synchronizationTimeHigh);
		result.append("\n");
		result.append("Synchronization time low: ");
		result.append(synchronizationTimeLow);
		result.append("\n");

		result.append("Num of cases with this node: ");
		result.append(freqCaseHasThisNode);
		result.append("\n");

		result.append("initialization frequencies: ");
		result.append(initializationFreq);
		result.append("\n");

		result.append("termination frequencies: ");
		result.append(terminationFreq);
		result.append("\n");

		result.append("Number of performer: ");
		result.append(numberOfPerformer);
		result.append("\n");

		return result.toString();
	}

	/**
	 * return true if current eventClassMapping contains eventClass
	 * 
	 * @param eventClass
	 * @return
	 */
	public boolean hasEventClass(XEventClass eventClass) {
		return eventClasses.contains(eventClass);
	}

	/**
	 * get the sum of frequency of event classes which refer to this node
	 * 
	 * @return
	 */
	public int getAggregatedEventClassFreq() {
		int total = 0;
		for (int subtotal : freqEventClasses) {
			total += subtotal;
		}
		return total;
	}

	/**
	 * @return the initializationFreq
	 */
	public int getInitializationFreq() {
		return initializationFreq;
	}

	/**
	 * @param initializationFreq
	 *            the initializationFreq to set
	 */
	public void setInitializationFreq(int initializationFreq) {
		this.initializationFreq = initializationFreq;
	}

	/**
	 * @return the terminationFreq
	 */
	public int getTerminationFreq() {
		return terminationFreq;
	}

	/**
	 * @param terminationFreq
	 *            the terminationFreq to set
	 */
	public void setTerminationFreq(int terminationFreq) {
		this.terminationFreq = terminationFreq;
	}

	/**
	 * increment number of times the node is executed last in a case
	 */
	public void incrementTerminationFreq() {
		terminationFreq++;
	}

	/**
	 * increment number of times the node is executed first in a case
	 */
	public void incrementInitializationFreq() {
		initializationFreq++;
	}

	/**
	 * decrement number of times the node is executed last in a case
	 */
	public void decrementTerminationFreq() {
		terminationFreq--;
	}

	/**
	 * decrement number of times the node is executed first in a case
	 */
	public void decrementInitializationFreq() {
		initializationFreq--;
	}

	/**
	 * @return the numberOfPerformer
	 */
	public int getNumberOfPerformer() {
		return numberOfPerformer;
	}

	/**
	 * @param numberOfPerformer
	 *            the numberOfPerformer to set
	 */
	public void setNumberOfPerformer(int numberOfPerformer) {
		this.numberOfPerformer = numberOfPerformer;
	}

	/**
	 * increment number of resources which perform an activity in this node
	 */
	public void incrementNumberOfPerformer() {
		numberOfPerformer++;
	}

	/**
	 * decrement number of resources which perform an activity in this node
	 */
	public void decrementNumberOfPerformer() {
		numberOfPerformer--;
	}

	/**
	 * @return the numberOfCases
	 */
	public int getNumberOfCases() {
		return numberOfCases;
	}

	/**
	 * @param numberOfCases
	 *            the numberOfCases to set
	 */
	public void setNumberOfCases(int numberOfCases) {
		this.numberOfCases = numberOfCases;
	}

	/**
	 * @return the maxFreqEventClass
	 */
	public int getMaxFreqEventClass() {
		return maxFreqEventClass;
	}

	/**
	 * @param maxFreqEventClass
	 *            the maxFreqEventClass to set
	 */
	public void setMaxFreqEventClass(int maxFreqEventClass) {
		this.maxFreqEventClass = maxFreqEventClass;
	}

	/**
	 * @return the divider
	 */
	public int getDivider() {
		return divider;
	}

	/**
	 * @param divider
	 *            the divider to set
	 */
	public void setDivider(int divider) {
		this.divider = divider;
	}

	/**
	 * GUI related methods
	 */
	public void decorate(Graphics2D g2d, double x, double y, double width, double height) {
		// fill in the semantic box
		GeneralPath path = new GeneralPath();

		// create semantic boxes
		double heightOfSplitBoxes = (height - (4 * PADDINGFROMBOXTOTEXT)) / 3;

		// general stroke
		BasicStroke stroke = new BasicStroke((float) 1.0);
		g2d.setStroke(stroke);

		/**
		 * JOIN boxes
		 */
		// give color
		g2d.setColor(new Color(50, 50, 50));
		double xForJoinBoxes = x + PADDINGFROMBOXTOTEXT;
		double yForJoinBoxes = y + (2 * PADDINGFROMBOXTOTEXT);
		java.awt.Shape[] joinBoxes = getBranchingIndicatorBoxes(xForJoinBoxes, yForJoinBoxes, heightOfSplitBoxes);
		path.append(joinBoxes[0], false);
		g2d.fill(path);
		path.reset();
		path.append(joinBoxes[1], false);
		g2d.fill(path);
		path.reset();
		path.append(joinBoxes[2], false);
		g2d.fill(path);
		g2d.drawString("AND", Math.round(xForJoinBoxes + BRANCHINGBOXWIDTH - 48),
				Math.round(y + PADDINGFROMBOXTOTEXT + TEXTHEIGHT));
		g2d.drawString("XOR", Math.round(xForJoinBoxes + BRANCHINGBOXWIDTH - 48),
				Math.round(y + (2 * PADDINGFROMBOXTOTEXT) + (2 * TEXTHEIGHT) + (3 * heightOfSplitBoxes)));

		/**
		 * SPLIT boxes
		 */
		double xForSplitBoxes = x + width - PADDINGFROMBOXTOTEXT - BRANCHINGBOXWIDTH;
		java.awt.Shape[] splitBoxes = getBranchingIndicatorBoxes(xForSplitBoxes, yForJoinBoxes, heightOfSplitBoxes);

		path.reset();
		path.append(splitBoxes[0], false);
		g2d.fill(path);
		path.reset();
		path.append(splitBoxes[1], false);
		g2d.fill(path);
		path.reset();
		path.append(splitBoxes[2], false);
		g2d.fill(path);
		g2d.drawString("AND", Math.round(xForSplitBoxes), Math.round(y + PADDINGFROMBOXTOTEXT + TEXTHEIGHT));
		g2d.drawString("XOR", Math.round(xForSplitBoxes),
				Math.round(y + (2 * PADDINGFROMBOXTOTEXT) + (2 * TEXTHEIGHT) + (3 * heightOfSplitBoxes)));

		/**
		 * Draw JOIN semantic indicator
		 */
		// give color
		g2d.setColor(getPerformanceColor(getWaitingTime(), getWaitingTimeLow(), getWaitingTimeHigh()));
		java.awt.Shape[] joinBoxContents = getBranchingBoxContents(xForJoinBoxes, yForJoinBoxes, heightOfSplitBoxes,
				getANDJOINrf(), getORJOINrf(), getXORJOINrf(), true);
		path.reset();
		path.append(joinBoxContents[0], false);
		g2d.fill(path);
		path.reset();
		path.append(joinBoxContents[1], false);
		g2d.fill(path);
		path.reset();
		path.append(joinBoxContents[2], false);
		g2d.fill(path);

		/**
		 * Draw SPLIT semantic indicator
		 */
		// give color
		g2d.setColor(getPerformanceColor(getSynchronizationTime(), getSynchronizationTimeLow(),
				getSynchronizationTimeHigh()));
		java.awt.Shape[] splitBoxContents = getBranchingBoxContents(xForSplitBoxes, yForJoinBoxes, heightOfSplitBoxes,
				getANDSPLITrf(), getORSPLITrf(), getXORSPLITrf(), false);
		path.reset();
		path.append(splitBoxContents[0], false);
		g2d.fill(path);
		path.reset();
		path.append(splitBoxContents[1], false);
		g2d.fill(path);
		path.reset();
		path.append(splitBoxContents[2], false);
		g2d.fill(path);

		/**
		 * Draw throughput
		 */
		g2d.setColor(getPerformanceColor(getAvgThroughput(), getThroughputTimeLow(), getThroughputTimeHigh()));
		path.reset();

		double throughputBoxHeight = (((double) getFreqCaseHasThisNode() / (double) getNumberOfCases()) * height);
		java.awt.Shape throughputFilling = new Rectangle2D.Double(x + BRANCHINGBOXWIDTH + (2 * PADDINGFROMBOXTOTEXT), y
				+ height - throughputBoxHeight, width - (2 * BRANCHINGBOXWIDTH) - (4 * PADDINGFROMBOXTOTEXT),
				throughputBoxHeight);
		path.append(throughputFilling, false);
		g2d.fill(path);

		// draw the stroke
		g2d.setColor(Color.BLACK);
		g2d.draw(path);

		path.reset();

		/**
		 * Label frequency and also throughput time
		 */
		// number of activity instances which is encountered in this node (note that this is different than activation frequency!)
		g2d.drawString(String.valueOf(getAggregatedEventClassFreq()),
				Math.round(x + BRANCHINGBOXWIDTH + (2 * PADDINGFROMBOXTOTEXT)) + 5, Math.round(y + height) - 5);

		// throughput time
		if (Double.compare(getAvgThroughput(), -1) == 0) {
			g2d.drawString("N/A", Math.round(x + width - BRANCHINGBOXWIDTH - (2 * PADDINGFROMBOXTOTEXT) - 23),
					Math.round(y + height) - 5);
		} else {
			FontMetrics fm = g2d.getFontMetrics();
			String text = nf.format(getAvgThroughput() / divider);
			g2d.drawString(text,
					Math.round(x + width - BRANCHINGBOXWIDTH - (2 * PADDINGFROMBOXTOTEXT) - fm.stringWidth(text)),
					Math.round(y + height) - 5);
		}

		// BVD: Added this code in order to correctly render HTML. 
		//
		// The label should be in a box from 
		// - Math.round(x + BRANCHINGBOXWIDTH + (2 * PADDINGFROMBOXTOTEXT) + 10), y to
		// - Math.round(x + width - (BRANCHINGBOXWIDTH + (2 * PADDINGFROMBOXTOTEXT) + 10), y+height)

		final int labelX = (int) Math.round(x + BRANCHINGBOXWIDTH + (2 * PADDINGFROMBOXTOTEXT) + 10);
		final int labelY = (int) Math.round(y);
		final int labelW = (int) Math.round(width - 2 * (BRANCHINGBOXWIDTH + (2 * PADDINGFROMBOXTOTEXT) + 10));
		final int labelH = (int) Math.round(height);

		JLabel label = new JLabel(getLabel());
		label.setPreferredSize(new Dimension(labelW, labelH));
		label.setSize(new Dimension(labelW, labelH));

		label.validate();
		label.paint(g2d.create(labelX, labelY, labelW, labelH));

		// REMOVED THIS CODE AS IT DOES NOT RENDER HTML
		//		// write label
		//		if (getLabel().length() > MAXLABELCHAR) {
		//			g2d.drawString(getLabel().substring(0, MAXLABELCHAR), Math.round(x + BRANCHINGBOXWIDTH
		//					+ (2 * PADDINGFROMBOXTOTEXT) + 10), Math.round(y + (height / 2)));
		//		} else {
		//			// estimation : 1 character has width 5
		//			g2d.drawString(getLabel(), Math.round(x + ((width / 2) - (5 * (getLabel().length() / 2)))), Math.round(y
		//					+ (height / 2)));
		//		}
	}

	/**
	 * return branching box of a node (AND, OR, XOR)
	 * 
	 * @param x
	 * @param y
	 * @param heightOfBoxes
	 * @param andFreq
	 * @param orFreq
	 * @param xorFreq
	 * @param isRightAligned
	 * @return
	 */
	private Shape[] getBranchingBoxContents(double x, double y, double heightOfBoxes, int andFreq, int orFreq,
			int xorFreq, boolean isRightAligned) {
		// result
		java.awt.Shape andShape;
		java.awt.Shape orShape;
		java.awt.Shape xorShape;

		// calculate relative length
		int totalFreq = andFreq + orFreq + xorFreq;
		double andWidth = ((double) andFreq / (double) totalFreq) * BRANCHINGBOXWIDTH;
		double orWidth = ((double) orFreq / (double) totalFreq) * BRANCHINGBOXWIDTH;
		double xorWidth = ((double) xorFreq / (double) totalFreq) * BRANCHINGBOXWIDTH;

		if (isRightAligned) {
			andShape = new Rectangle2D.Double(x + BRANCHINGBOXWIDTH - andWidth, y, andWidth, heightOfBoxes);
			orShape = new Rectangle2D.Double(x + BRANCHINGBOXWIDTH - orWidth, y + heightOfBoxes, orWidth, heightOfBoxes);
			xorShape = new Rectangle2D.Double(x + BRANCHINGBOXWIDTH - xorWidth, y + (2 * heightOfBoxes), xorWidth,
					heightOfBoxes);
		} else {
			andShape = new Rectangle2D.Double(x, y, andWidth, heightOfBoxes);
			orShape = new Rectangle2D.Double(x, y + heightOfBoxes, orWidth, heightOfBoxes);
			xorShape = new Rectangle2D.Double(x, y + (2 * heightOfBoxes), xorWidth, heightOfBoxes);
		}

		// result
		return new Shape[] { andShape, orShape, xorShape };
	}

	/**
	 * return performance color based on timeLow and timeHigh as lower and upper
	 * bound, respectively
	 * 
	 * @param time
	 * @param timeLow
	 * @param timeHigh
	 * @return
	 */
	private Color getPerformanceColor(double time, double timeLow, double timeHigh) {
		if (time <= timeLow) {
			return Color.GREEN;
		} else if (time <= timeHigh) {
			return Color.YELLOW;
		} else {
			return Color.RED;
		}
	}

	/**
	 * return branching indicator box
	 * 
	 * @param x
	 * @param y
	 * @param heightOfSplitBoxes
	 * @return
	 */
	public java.awt.Shape[] getBranchingIndicatorBoxes(double x, double y, double heightOfSplitBoxes) {
		java.awt.Shape[] result = new java.awt.Shape[3];

		java.awt.Shape AND = new Rectangle2D.Double(x, y, BRANCHINGBOXWIDTH, heightOfSplitBoxes);
		java.awt.Shape OR = new Rectangle2D.Double(x, y + heightOfSplitBoxes, BRANCHINGBOXWIDTH, heightOfSplitBoxes);
		java.awt.Shape XOR = new Rectangle2D.Double(x, y + (2 * heightOfSplitBoxes), BRANCHINGBOXWIDTH,
				heightOfSplitBoxes);

		result[0] = AND;
		result[1] = OR;
		result[2] = XOR;

		return result;
	}

	/**
	 * method to update node size after there is any modification to its
	 * frequency value
	 */
	public void updateNodeSize() {
		double scaledHeight = 0;
		if ((getMaxFreqEventClass() > 0) && (getAggregatedEventClassFreq() > 0)) { // height should be depend on relative frequency
			scaledHeight = STANDARDHEIGHT * ((double) getAggregatedEventClassFreq() / (double) getMaxFreqEventClass());
		}
		getAttributeMap().remove(AttributeMap.SIZE);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(STANDARDWIDTH, (int) (scaledHeight + MINHEIGHT))); // default value
	}

	/**
	 * @return the activationFreq
	 */
	public int getActivationFreq() {
		return activationFreq;
	}

	/**
	 * @param activationFreq
	 *            the activationFreq to set
	 */
	public void setActivationFreq(int activationFreq) {
		this.activationFreq = activationFreq;
	}

	/**
	 * increment number of times where there is an event which correspond to
	 * this node
	 */
	public void incrementActivationFreq() {
		activationFreq++;
	}

	/**
	 * decrement number of times where there is an event which correspond to
	 * this node
	 */
	public void decrementActivationFreq() {
		activationFreq--;
	}

	public void setToolTip(String newTooltip) {
		getAttributeMap().remove(AttributeMap.TOOLTIP);
		getAttributeMap().put(AttributeMap.TOOLTIP, newTooltip);
	}
}
